# -*- coding: utf-8 -*-
"""
Spyder Editor

This is a temporary script file.
"""
#%%
import os
import igl
import numpy as np
import pandas as pd
from pathlib import Path
#%%
def findObjPaths(directory):
    list = []
    for root, dirs, files in os.walk(directory):
        for file in files:
            if(file.endswith(".obj")):
                list.append(os.path.join(root, file))
    return list

def computeMean(pathToFolder : str):
    listPath = findObjPaths(pathToFolder)
    return computeMean(listPath)

def computeMean(listPath : list):
    v_bar, f_bar = igl.read_triangle_mesh(listPath[0])
    N = len(v_bar)
    M = len(listPath)
    print(M)
    i = 0
    for obj in listPath:
        v_i, f_i = igl.read_triangle_mesh(obj)
        v_bar = v_bar + v_i
        if( i%10 ==0):
            print(i%M)
        i= i+1
    v_bar = v_bar / M
    return v_bar,f_bar

def computeMean(arrays :np.array):
    v_bar = np.array(arrays[0])
    M = np.shape(arrays)[0]
    print(M)
    for v_i in arrays:
        v_bar = v_bar + v_i
    v_bar = v_bar / M
    return v_bar

def readPositions(listPath: list):
    print("readPositions()")
    v_bar, f_bar = igl.read_triangle_mesh(listPath[0])
    N = len(v_bar)
    M = len(listPath)
    print(M)
    i = 0
    df = pd.DataFrame()
    print(df)
    for obj in listPath:
        objPath = Path(obj)
        v_i, f_i = igl.read_triangle_mesh(obj)
        expressionID = str(os.path.basename(objPath)).split(".")[0]
        if not expressionID in df.columns:
            df[expressionID] = ""
            df[expressionID] = df[expressionID].astype(object)
        df.at[str(os.path.basename(objPath.parents[1])),expressionID]=[v_i]
        if( i%10 ==0):
            print(i%M)
        i= i+1
    return df
    
    
def computeUis(pathToFolder):
    listPath = findObjPaths(pathToFolder)
    u_is = np.zeros(shape=(0,N*3))
    v_bar,f_bar = computeMean(listPath)
    for obj in listPath:
        v_i, f_i = igl.read_triangle_mesh(obj)
        ui = v_bar - v_i
        ui = ui.flatten(order='F').reshape(1,-1)
        u_is = np.append(u_is , ui, axis = 0 )
    return u_is

#%%

def computeKdirs(pathToFolder):
    
    k_dirs = np.zeros(shape=(0,N*12))
    
    for obj in listPath:
        v_i, f_i = igl.read_triangle_mesh(obj)
        v1, v2, k1, k2 = igl.principal_curvature(v_i, f_i)
        v1v1t = np.zeros((np.shape(v1)[0],6))
        v2v2t = np.zeros((np.shape(v1)[0],6))
        for i in range(0, np.shape(v1)[0]):
            tmp = v1[i].reshape(3,1)
            vivit = tmp.dot(tmp.transpose())
            vivit = vivit[np.triu_indices(3)] 
            v1v1t[i] = vivit
        for i in range(0, np.shape(v2)[0]): 
            tmp = v2[i].reshape(3,1)
            vivit = tmp.dot(tmp.transpose())
            vivit = vivit[np.triu_indices(3)] 
            v2v2t[i] = vivit
            
        
        vs = np.append(v1v1t , v2v2t, axis = 1 )
        vs = vs.flatten(order='F').reshape((1,-1))
    
        k_dirs = np.append(k_dirs , vs, axis = 0 )
        
    return k_dirs


def computeK1K2s(pathToFolder):

    k1k2s = np.zeros(shape=(0,N*2))
    
    for obj in listPath:
        v_i, f_i = igl.read_triangle_mesh(obj)
        v1, v2, k1, k2 = igl.principal_curvature(v_i, f_i)
        k1 = np.reshape(k1,(-1,1))
        k2 = np.reshape(k2,(-1,1))
        k1k2 = np.append(k1 , k2, axis = 1 )
        k1k2 = k1k2.flatten(order='F').reshape((1,-1))
        k1k2s = np.append(k1k2s , k1k2, axis = 0 )
        
    return k1k2s





from qojpca import base
import numpy as np
import pandas as pd 
from numpy import linalg as LA
import plotly.graph_objects as go
import os

def main():
    print(os.getcwd())
    folder = "output_lambda_0/"
    P=np.load(folder+"P.npy")
    P_vals=np.load(folder+"P_vals.npy").reshape(-1,1)

    Q=np.load(folder+"Q.npy")
    Q_vals=np.load(folder+"Q_vals.npy").reshape(-1,1)

    folder = "output_lambda_10/"
    P_ortho_10=np.load(folder+"P.npy")
    P_ortho_vals_10=np.load(folder+"P_vals.npy").reshape(-1,1)


    Q_ortho_10=np.load(folder+"Q.npy")
    Q_ortho_vals_10=np.load(folder+"Q_vals.npy").reshape(-1,1)

    folder = "output_lambda_100/"

    P_ortho_100=np.load(folder+"P.npy")
    P_ortho_vals_100=np.load(folder+"P_vals.npy").reshape(-1,1)


    Q_ortho_100=np.load(folder+"Q.npy")
    Q_ortho_vals_100=np.load(folder+"Q_vals.npy").reshape(-1,1)

    folder = "output_lambda_500/"

    P_ortho_500=np.load(folder+"P.npy")
    P_ortho_vals_500=np.load(folder+"P_vals.npy").reshape(-1,1)


    Q_ortho_500=np.load(folder+"Q.npy")
    Q_ortho_vals_500=np.load(folder+"Q_vals.npy").reshape(-1,1)

    folder = "output_lambda_1000/"

    P_ortho_1000=np.load(folder+"P.npy")
    P_ortho_vals_1000=np.load(folder+"P_vals.npy").reshape(-1,1)


    Q_ortho_1000=np.load(folder+"Q.npy")
    Q_ortho_vals_1000=np.load(folder+"Q_vals.npy").reshape(-1,1)

    results = pd.DataFrame(columns=["Number of modes", "Orthogonality score", "type"])
    print(P.shape)
    for i in range(1,50):
        score1 = np.linalg.norm(P[:i,:].dot(Q[:i,:].transpose()))
        score2 = np.linalg.norm(P_ortho_10.dot(Q_ortho_10[:i,:].transpose()))
        score3 = np.linalg.norm(P_ortho_100.dot(Q_ortho_100[:i,:].transpose()))
        score4 = np.linalg.norm(P_ortho_500.dot(Q_ortho_500[:i,:].transpose()))
        score5 = np.linalg.norm(P_ortho_1000.dot(Q_ortho_1000[:i,:].transpose()))

        results=results.append({"Number of modes": i,"Orthogonality score" :score1, "type": "Joint PCA" },  ignore_index=True)
        results=results.append({"Number of modes": i,"Orthogonality score" :score2, "type": "Regularized joint PCA 10" }, ignore_index=True)
        results=results.append({"Number of modes": i,"Orthogonality score" :score3, "type": "Regularized joint PCA 100" }, ignore_index=True)    
        results=results.append({"Number of modes": i,"Orthogonality score" :score4, "type": "Regularized joint PCA 500" }, ignore_index=True)   
        results=results.append({"Number of modes": i,"Orthogonality score" :score5, "type": "Regularized joint PCA 1000" }, ignore_index=True)   
    np.random.seed(1)

    N = 100
    random_x = np.linspace(0, 1, N)
    random_y0 = np.random.randn(N) + 5
    random_y1 = np.random.randn(N)
    random_y2 = np.random.randn(N) - 5

    # Create traces
    fig = go.Figure()
    fig.add_trace(go.Scatter(y=results.loc[results['type']=='Joint PCA']['Orthogonality score'],
                             x=results.loc[results['type']=='Joint PCA']['Number of modes'],
                        mode='lines+markers',
                                name=r"$\text{Joint PCA}..............$"))
    fig.add_trace(go.Scatter(y=results.loc[results['type']=='Regularized joint PCA 10']['Orthogonality score'],
                             x=results.loc[results['type']=='Regularized joint PCA 10']['Number of modes'],
                        mode='lines+markers',
                        name=r"$\text{QOJPCA}\;\lambda = 10 e_1 $)"))
    fig.add_trace(go.Scatter(y=results.loc[results['type']=='Regularized joint PCA 100']['Orthogonality score'] ,
                             x=results.loc[results['type']=='Regularized joint PCA 100']['Number of modes'],
                        mode='lines+markers',
                        name=r"$\text{QOJPCA}\;\lambda = 100 e_1  \quad      $)"))
    # fig.add_trace(go.Scatter(y=results.loc[results['type']=='Regularized joint PCA 500']['Orthogonality score'] ,
    #                          x=results.loc[results['type']=='Regularized joint PCA 500']['Number of modes'],
    #                     mode='lines+markers',
    #                     name=r"$\text{QOJPCA}\;500\lambda  \quad      $)"))
    fig.add_trace(go.Scatter(y=results.loc[results['type']=='Regularized joint PCA 1000']['Orthogonality score'] ,
                             x=results.loc[results['type']=='Regularized joint PCA 1000']['Number of modes'],
                        mode='lines+markers',
                        name=r"$\text{QOJPCA}\;\lambda = 1000 e_1  \quad      $)"))
    fig.update_layout(
        #title='Mean Error (mm) before expression correction on the test set',
        font_color="black",
        font_family="sans serif",
        font_size=25,
        margin=dict(
            l=40,
            r=30,
            b=30,
            t=50,
        ),
        legend=dict(
            x=.7,
            y=0.9,
            traceorder="normal",
            font=dict(
                family="sans serif",
                size=50,
                color="black"
            ),
        ),
        paper_bgcolor='rgb(250, 250, 250)',
        plot_bgcolor='rgb(250, 250, 250)',
        showlegend=True,xaxis_title="Number of components",
        yaxis_title=r"$\Large{||PQ^T||}$",
        )
    fig.show()
if __name__ == "__main__":  # pragma: no cover
    main()
